@@ -19,11 +19,14 @@ use crate::isocode::{IsoCode639_1, IsoCode639_3};
 use serde::{Deserialize, Serialize};
 use std::collections::HashSet;
 use strum::IntoEnumIterator;
-use strum_macros::EnumIter;
+use strum_macros::{EnumIter, EnumString};
 
 /// This enum specifies the so far 75 supported languages which can be detected by *Lingua*.
-#[derive(Clone, Debug, Serialize, Deserialize, EnumIter, Eq, PartialEq, Hash, Ord, PartialOrd)]
+#[derive(
+    Clone, Debug, Serialize, Deserialize, EnumIter, Eq, PartialEq, Hash, Ord, PartialOrd, EnumString,
+)]
 #[serde(rename_all(serialize = "UPPERCASE", deserialize = "UPPERCASE"))]
+#[strum(ascii_case_insensitive)]
 pub enum Language {
     Afrikaans,
     Albanian,
@@ -421,6 +424,7 @@ impl Language {
 mod tests {
     use super::*;
     use crate::language::Language::*;
+    use std::str::FromStr;
 
     #[test]
     fn test_language_serializer() {
@@ -434,6 +438,12 @@ mod tests {
         assert_eq!(deserialized, Language::English);
     }
 
+    #[test]
+    fn test_from_str() {
+        let language = Language::from_str("english").unwrap();
+        assert_eq!(language, Language::English);
+    }
+
     #[test]
     fn assert_all_languages_are_available() {
         assert_eq!(
